@@ -21,6 +21,7 @@ from six.moves.urllib import parse
 try:
     from ws4py.client import WebSocketBaseClient
     from ws4py.manager import WebSocketManager
+    from ws4py.messaging import BinaryMessage
     _ws4py_installed = True
 except ImportError:  # pragma: no cover
     WebSocketBaseClient = object
@@ -29,6 +30,7 @@ except ImportError:  # pragma: no cover
 from pylxd import managers
 from pylxd.exceptions import LXDAPIException
 from pylxd.models import _model as model
+from pylxd.models.operation import Operation
 
 if six.PY2:
     # Python2.7 doesn't have this natively
@@ -67,6 +69,7 @@ class Container(model.Model):
     profiles = model.Attribute()
     status = model.Attribute(readonly=True)
     last_used_at = model.Attribute(readonly=True)
+    location = model.Attribute(readonly=True)
 
     status_code = model.Attribute(readonly=True)
     stateful = model.Attribute(readonly=True)
@@ -138,12 +141,10 @@ class Container(model.Model):
             """File deletion is an extension API and may not be available.
             https://github.com/lxc/lxd/blob/master/doc/api-extensions.md#file_delete
             """
-            return u'file_delete' in self._client.host_info['api_extensions']
+            return self._client.has_api_extension('file_delete')
 
         def delete(self, filepath):
-            if not self.delete_available():
-                raise ValueError(
-                    'File Deletion is not available for this host')
+            self._client.assert_has_api_extension('file_delete')
             response = self._client.api.containers[
                 self._container.name].files.delete(
                 params={'path': filepath})
@@ -257,9 +258,22 @@ class Container(model.Model):
         return containers
 
     @classmethod
-    def create(cls, client, config, wait=False):
-        """Create a new container config."""
-        response = client.api.containers.post(json=config)
+    def create(cls, client, config, wait=False, target=None):
+        """Create a new container config.
+
+        :param client: client instance
+        :type client: Client
+        :param config: The configuration for the new container.
+        :type config: dict
+        :param wait: Whether to wait for async operations to complete.
+        :type wait: bool
+        :param target: If in cluster mode, the target member.
+        :type target: str
+        :raises LXDAPIException: if something goes wrong.
+        :returns: a container if successful
+        :rtype: :class:`Container`
+        """
+        response = client.api.containers.post(json=config, target=target)
 
         if wait:
             client.operations.wait_for_operation(response.json()['operation'])
@@ -336,7 +350,11 @@ class Container(model.Model):
                                force=force,
                                wait=wait)
 
-    def execute(self, commands, environment={}, encoding=None, decode=True):
+    def execute(
+            self, commands, environment={}, encoding=None, decode=True,
+            stdin_payload=None, stdin_encoding="utf-8",
+            stdout_handler=None, stderr_handler=None
+    ):
         """Execute a command on the container.
 
         In pylxd 2.2, this method will be renamed `execute` and the existing
@@ -353,6 +371,16 @@ class Container(model.Model):
         :param decode: Whether to decode the stdout/stderr or just return the
             raw buffers.
         :type decode: bool
+        :param stdin_payload: Payload to pass via stdin
+        :type stdin_payload: Can be a file, string, bytearray, generator or
+            ws4py Message object
+        :param stdin_encoding: Encoding to pass text to stdin (default utf-8)
+        :param stdout_handler: Callable than receive as first parameter each
+            message recived via stdout
+        :type stdout_handler: Callable[[str], None]
+        :param stderr_handler: Callable than receive as first parameter each
+            message recived via stderr
+        :type stderr_handler: Callable[[str], None]
         :raises ValueError: if the ws4py library is not installed.
         :returns: The return value, stdout and stdin
         :rtype: _ContainerExecuteResult() namedtuple
@@ -370,22 +398,26 @@ class Container(model.Model):
         })
 
         fds = response.json()['metadata']['metadata']['fds']
-        operation_id = response.json()['operation'].split('/')[-1]
+        operation_id = \
+            Operation.extract_operation_id(response.json()['operation'])
         parsed = parse.urlparse(
             self.client.api.operations[operation_id].websocket._api_endpoint)
 
         with managers.web_socket_manager(WebSocketManager()) as manager:
-            stdin = _StdinWebsocket(self.client.websocket_url)
+            stdin = _StdinWebsocket(
+                self.client.websocket_url, payload=stdin_payload,
+                encoding=stdin_encoding
+            )
             stdin.resource = '{}?secret={}'.format(parsed.path, fds['0'])
             stdin.connect()
             stdout = _CommandWebsocketClient(
                 manager, self.client.websocket_url,
-                encoding=encoding, decode=decode)
+                encoding=encoding, decode=decode, handler=stdout_handler)
             stdout.resource = '{}?secret={}'.format(parsed.path, fds['1'])
             stdout.connect()
             stderr = _CommandWebsocketClient(
                 manager, self.client.websocket_url,
-                encoding=encoding, decode=decode)
+                encoding=encoding, decode=decode, handler=stderr_handler)
             stderr.resource = '{}?secret={}'.format(parsed.path, fds['2'])
             stderr.connect()
 
@@ -401,14 +433,48 @@ class Container(model.Model):
             while len(manager.websockets.values()) > 0:
                 time.sleep(.1)  # pragma: no cover
 
-            stdout.close()
-            stderr.close()
             manager.stop()
             manager.join()
 
             return _ContainerExecuteResult(
                 operation.metadata['return'], stdout.data, stderr.data)
 
+    def raw_interactive_execute(self, commands, environment=None):
+        """Execute a command on the container interactively and returns
+        urls to websockets. The urls contain a secret uuid, and can be accesses
+        without further authentication. The caller has to open and manage
+        the websockets themselves.
+
+        :param commands: The command and arguments as a list of strings
+           (most likely a shell)
+        :type commands: [str]
+        :param environment: The environment variables to pass with the command
+        :type environment: {str: str}
+        :returns: Two urls to an interactive websocket and a control socket
+        :rtype: {'ws':str,'control':str}
+        """
+        if isinstance(commands, six.string_types):
+            raise TypeError("First argument must be a list.")
+
+        if environment is None:
+            environment = {}
+
+        response = self.api['exec'].post(json={
+            'command': commands,
+            'environment': environment,
+            'wait-for-websocket': True,
+            'interactive': True,
+        })
+
+        fds = response.json()['metadata']['metadata']['fds']
+        operation_id = response.json()['operation']\
+            .split('/')[-1].split('?')[0]
+        parsed = parse.urlparse(
+            self.client.api.operations[operation_id].websocket._api_endpoint)
+
+        return {'ws': '{}?secret={}'.format(parsed.path, fds['0']),
+                'control': '{}?secret={}'.format(parsed.path, fds['control'])}
+
     def migrate(self, new_client, wait=False):
         """Migrate a container.
 
@@ -422,8 +488,21 @@ class Container(model.Model):
         if self.api.scheme in ('http+unix',):
             raise ValueError('Cannot migrate from a local client connection')
 
-        return new_client.containers.create(
-            self.generate_migration_data(), wait=wait)
+        if self.status_code == 103:
+            try:
+                res = new_client.containers.create(
+                    self.generate_migration_data(), wait=wait)
+            except LXDAPIException as e:
+                if e.response.status_code == 103:
+                    self.delete()
+                    return new_client.containers.get(self.name)
+                else:
+                    raise e
+        else:
+            res = new_client.containers.create(
+                self.generate_migration_data(), wait=wait)
+        self.delete()
+        return res
 
     def generate_migration_data(self):
         """Generate the migration data.
@@ -489,6 +568,7 @@ class _CommandWebsocketClient(WebSocketBaseClient):  # pragma: no cover
         self.manager = manager
         self.decode = kwargs.pop('decode', True)
         self.encoding = kwargs.pop('encoding', None)
+        self.handler = kwargs.pop('handler', None)
         self.message_encoding = None
         super(_CommandWebsocketClient, self).__init__(*args, **kwargs)
 
@@ -497,14 +577,19 @@ class _CommandWebsocketClient(WebSocketBaseClient):  # pragma: no cover
         self.buffer = []
 
     def received_message(self, message):
+        if message.data is None or len(message.data) == 0:
+            self.manager.remove(self)
+            return
         if message.encoding and self.message_encoding is None:
             self.message_encoding = message.encoding
+        if self.handler:
+            self.handler(self._maybe_decode(message.data))
         self.buffer.append(message.data)
+        if isinstance(message, BinaryMessage):
+            self.manager.remove(self)
 
-    @property
-    def data(self):
-        buffer = b''.join(self.buffer)
-        if self.decode:
+    def _maybe_decode(self, buffer):
+        if self.decode and buffer is not None:
             if self.encoding:
                 return buffer.decode(self.encoding)
             if self.message_encoding:
@@ -513,17 +598,38 @@ class _CommandWebsocketClient(WebSocketBaseClient):  # pragma: no cover
             return buffer.decode('utf-8')
         return buffer
 
+    @property
+    def data(self):
+        buffer = b''.join(self.buffer)
+        return self._maybe_decode(buffer)
+
 
 class _StdinWebsocket(WebSocketBaseClient):  # pragma: no cover
     """A websocket client for handling stdin.
 
-    The nature of stdin in Container.execute means that we don't
-    ever use this connection. It is closed as soon as it completes
-    the handshake.
+    Allow comunicate with container commands via stdin
     """
 
+    def __init__(self, url, payload=None, **kwargs):
+        self.encoding = kwargs.pop('encoding', None)
+        self.payload = payload
+        super(_StdinWebsocket, self).__init__(url, **kwargs)
+
+    def _smart_encode(self, msg):
+        if type(msg) == six.text_type and self.encoding:
+            return msg.encode(self.encoding)
+        return msg
+
     def handshake_ok(self):
-        self.close()
+        if self.payload:
+            if hasattr(self.payload, "read"):
+                self.send(
+                    (self._smart_encode(line) for line in self.payload),
+                    binary=True
+                )
+            else:
+                self.send(self._smart_encode(self.payload), binary=True)
+        self.send(b"", binary=False)
 
 
 class Snapshot(model.Model):
